@@ -1,37 +1,5 @@
-/*
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MIT
- *
- * Portions created by Alan Antonuk are Copyright (c) 2012-2013
- * Alan Antonuk. All Rights Reserved.
- *
- * Portions created by VMware are Copyright (c) 2007-2012 VMware, Inc.
- * All Rights Reserved.
- *
- * Portions created by Tony Garnock-Jones are Copyright (c) 2009-2010
- * VMware, Inc. and Tony Garnock-Jones. All Rights Reserved.
- *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use, copy,
- * modify, merge, publish, distribute, sublicense, and/or sell copies
- * of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- * ***** END LICENSE BLOCK *****
- */
+// Copyright 2007 - 2021, Alan Antonuk and the rabbitmq-c contributors.
+// SPDX-License-Identifier: mit
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -47,9 +15,11 @@
 
 #define INITIAL_ARRAY_SIZE 16
 #define INITIAL_TABLE_SIZE 16
+#define TABLE_DEPTH_LIMIT 100
 
 static int amqp_decode_field_value(amqp_bytes_t encoded, amqp_pool_t *pool,
-                                   amqp_field_value_t *entry, size_t *offset);
+                                   amqp_field_value_t *entry, size_t *offset,
+                                   int depth);
 
 static int amqp_encode_field_value(amqp_bytes_t encoded,
                                    amqp_field_value_t *entry, size_t *offset);
@@ -57,7 +27,7 @@ static int amqp_encode_field_value(amqp_bytes_t encoded,
 /*---------------------------------------------------------------------------*/
 
 static int amqp_decode_array(amqp_bytes_t encoded, amqp_pool_t *pool,
-                             amqp_array_t *output, size_t *offset) {
+                             amqp_array_t *output, size_t *offset, int depth) {
   uint32_t arraysize;
   int num_entries = 0;
   int allocated_entries = INITIAL_ARRAY_SIZE;
@@ -93,7 +63,8 @@ static int amqp_decode_array(amqp_bytes_t encoded, amqp_pool_t *pool,
       entries = newentries;
     }
 
-    res = amqp_decode_field_value(encoded, pool, &entries[num_entries], offset);
+    res = amqp_decode_field_value(encoded, pool, &entries[num_entries], offset,
+                                  depth);
     if (res < 0) {
       goto out;
     }
@@ -122,8 +93,9 @@ out:
   return res;
 }
 
-int amqp_decode_table(amqp_bytes_t encoded, amqp_pool_t *pool,
-                      amqp_table_t *output, size_t *offset) {
+static int amqp_decode_table_internal(amqp_bytes_t encoded, amqp_pool_t *pool,
+                                      amqp_table_t *output, size_t *offset,
+                                      int depth) {
   uint32_t tablesize;
   int num_entries = 0;
   amqp_table_entry_t *entries;
@@ -173,7 +145,7 @@ int amqp_decode_table(amqp_bytes_t encoded, amqp_pool_t *pool,
     }
 
     res = amqp_decode_field_value(encoded, pool, &entries[num_entries].value,
-                                  offset);
+                                  offset, depth);
     if (res < 0) {
       goto out;
     }
@@ -202,10 +174,20 @@ out:
   return res;
 }
 
+int amqp_decode_table(amqp_bytes_t encoded, amqp_pool_t *pool,
+                      amqp_table_t *output, size_t *offset) {
+  return amqp_decode_table_internal(encoded, pool, output, offset, 0);
+}
+
 static int amqp_decode_field_value(amqp_bytes_t encoded, amqp_pool_t *pool,
-                                   amqp_field_value_t *entry, size_t *offset) {
+                                   amqp_field_value_t *entry, size_t *offset,
+                                   int depth) {
   int res = AMQP_STATUS_BAD_AMQP_DATA;
 
+  if (depth > TABLE_DEPTH_LIMIT) {
+    return AMQP_STATUS_BAD_AMQP_DATA;
+  }
+
   if (!amqp_decode_8(encoded, offset, &entry->kind)) {
     goto out;
   }
@@ -274,14 +256,16 @@ static int amqp_decode_field_value(amqp_bytes_t encoded, amqp_pool_t *pool,
     }
 
     case AMQP_FIELD_KIND_ARRAY:
-      res = amqp_decode_array(encoded, pool, &(entry->value.array), offset);
+      res = amqp_decode_array(encoded, pool, &(entry->value.array), offset,
+                              depth + 1);
       goto out;
 
     case AMQP_FIELD_KIND_TIMESTAMP:
       TRIVIAL_FIELD_DECODER(64);
 
     case AMQP_FIELD_KIND_TABLE:
-      res = amqp_decode_table(encoded, pool, &(entry->value.table), offset);
+      res = amqp_decode_table_internal(encoded, pool, &(entry->value.table),
+                                       offset, depth + 1);
       goto out;
 
     case AMQP_FIELD_KIND_VOID:
