@@ -4,7 +4,7 @@ use warnings;
 
 use Net::AMQP::RabbitMQ;
 use Test::More ();
-use Carp qw/carp/;
+use Carp       qw/carp/;
 
 sub new {
   my ( $class, %options ) = @_;
@@ -12,12 +12,9 @@ sub new {
   my $mq     = Net::AMQP::RabbitMQ->new;
   my $unique = _unique();
 
-  my $ssl = $ENV{MQSSL} ? 1 : 0;
-  my $ssl_cacert =
-    exists $ENV{MQSSLCACERT}
-    ? $ENV{MQSSLCACERT}
-    : "t/ssl/api-cloudamqp-com-chain.pem",
-    my $ssl_verify_host = 1;
+  my $ssl             = $ENV{MQSSL} ? 1 : 0;
+  my $ssl_cacert      = $ENV{MQSSLCACERT};
+  my $ssl_verify_host = 1;
   if ( defined( $ENV{MQSSLVERIFYHOST} ) ) {
     $ssl_verify_host = $ENV{MQSSLVERIFYHOST};
   }
@@ -32,11 +29,12 @@ sub new {
     $ssl_verify_peer = $ENV{MQSSLVERIFYPEER};
   }
 
+  # THESE VALUES MUST BE USER-SUPPLIED!
   my $port;
-  my $host     = "shrimp.rmq.cloudamqp.com";
-  my $username = "frkwiwbi";
-  my $password = "n1rN3wmzelie8TYCTRjK9KHnJxo10HyN";
-  my $vhost    = "frkwiwbi";
+  my $host     = "";
+  my $username = "";
+  my $password = "";
+  my $vhost    = "";
 
   if ( $ssl || $options{ssl} ) {
     Test::More::note("ssl mode");
@@ -45,7 +43,7 @@ sub new {
     $username = $ENV{MQSSLUSERNAME} if exists $ENV{MQSSLUSERNAME};
     $password = $ENV{MQSSLPASSWORD} if exists $ENV{MQSSLPASSWORD};
     $vhost    = $ENV{MQSSLVHOST}    if exists $ENV{MQSSLVHOST};
-    $port     = $ENV{MQSSLPORT} || 5671;
+    $port     = exists $ENV{MQSSLPORT} ? $ENV{MQSSLPORT} : undef;
   }
   else {
     Test::More::note("non-ssl mode");
@@ -53,11 +51,16 @@ sub new {
     $username = $ENV{MQUSERNAME} if exists $ENV{MQUSERNAME};
     $password = $ENV{MQPASSWORD} if exists $ENV{MQPASSWORD};
     $vhost    = $ENV{MQVHOST}    if exists $ENV{MQVHOST};
-    $port     = $ENV{MQPORT} || 5672;
+    $port     = exists $ENV{MQPORT} ? $ENV{MQPORT} : undef;
   }
   my $admin_protocol = $ENV{MQADMINPROTOCOL} || "https";
   my $admin_port     = $ENV{MQADMINPORT}     || "443";
 
+  if ( !defined $host || !defined $username ) {
+    die
+'No host or user defined. Please see the https://metacpan.org/pod/Net::AMQP::RabbitMQ#RUNNING-THE-TEST-SUITE for more information.';
+  }
+
   #hack but it's ok as it's for testing and I don't want more deps
   my $uri_encoded_vhost = $vhost;
   $uri_encoded_vhost =~ s|/|%2F|g;
@@ -124,7 +127,6 @@ sub connect {
   my $options = {
     user            => $self->{username},
     password        => $self->{password},
-    port            => $self->{port},
     ssl             => $self->{ssl},
     ssl_verify_host => $self->{ssl_verify_host},
     ssl_verify_peer => $self->{ssl_verify_peer},
@@ -132,6 +134,9 @@ sub connect {
     ssl_init        => $self->{ssl_init},
     vhost           => $self->{vhost},
   };
+  if ( defined $self->{port} ) {
+    $options->{port} = $self->{port};
+  }
   if ( defined $heartbeat ) {
     $options->{heartbeat} = $heartbeat;
   }
@@ -149,10 +154,9 @@ sub connect {
 sub get_connection_options {
   my ($self) = @_;
 
-  return {
+  my $to_return = {
     user            => $self->{username},
     password        => $self->{password},
-    port            => $self->{port},
     ssl             => $self->{ssl},
     ssl_verify_host => $self->{ssl_verify_host},
     ssl_verify_peer => $self->{ssl_verify_peer},
@@ -161,6 +165,11 @@ sub get_connection_options {
     vhost           => $self->{vhost},
   };
 
+  if ( defined $self->{port} ) {
+    $to_return->{port} = $self->{port};
+  }
+
+  return $to_return;
 }
 
 sub heartbeat {
